@@ -1,11 +1,9 @@
-#![cfg_attr(all(test, feature = "nightly"), feature(i128, i128_type))]
+#![cfg_attr(not(test), no_std)]
 #![cfg_attr(all(feature = "nightly"), feature(const_fn, allow_internal_unstable, macro_vis_matcher))]
 #![forbid(missing_docs)]
 
-//! A library for defining enums that can be used in compact bit sets.
-//!
-//! It supports enums up to 64 variants on stable Rust, and up to 128 variants on nightly Rust with
-//! the `nightly` feature enabled.
+//! A library for defining enums that can be used in compact bit sets. It supports enums up to 128
+//! variants, and has a macro to use these sets in constants.
 //!
 //! # Defining enums for use with EnumSet
 //!
@@ -52,7 +50,7 @@
 //! #     enum Enum { A, B, C }
 //! # }
 //! # fn main() {
-//! const CONST_SET: EnumSet<Enum> = enum_set!(Enum, Enum::A | Enum::B);
+//! const CONST_SET: EnumSet<Enum> = enum_set!(Enum::A | Enum::B);
 //! assert_eq!(CONST_SET, Enum::A | Enum::B);
 //! # }
 //! ```
@@ -82,10 +80,13 @@
 //! [`enum_set!`]: ./macro.enum_set.html
 //! [`enum_set_type!`]: ./macro.enum_set_type.html
 
-use std::fmt;
-use std::fmt::{Debug, Formatter};
-use std::hash::Hash;
-use std::ops::*;
+#[cfg(test)]
+extern crate core;
+
+use core::fmt;
+use core::fmt::{Debug, Formatter};
+use core::hash::Hash;
+use core::ops::*;
 
 #[doc(hidden)]
 pub trait EnumSetType : Copy {
@@ -100,6 +101,15 @@ pub trait EnumSetType : Copy {
     fn count_ones(val: Self::Repr) -> usize;
     fn into_u8(self) -> u8;
     fn from_u8(val: u8) -> Self;
+
+    fn repr_to_u128(bits: Self::Repr) -> u128;
+    fn repr_from_u128(bits: u128) -> Self::Repr;
+}
+
+#[doc(hidden)]
+pub struct EnumSetSameTypeHack<'a, T: EnumSetType + 'static> {
+    pub unified: &'a [T],
+    pub enum_set: EnumSet<T>,
 }
 
 /// An efficient set type for enums created with the [`enum_set_type!`](./macro.enum_set_type.html)
@@ -128,6 +138,21 @@ impl <T : EnumSetType> EnumSet<T> {
         EnumSet { __enumset_underlying: T::ZERO }
     }
 
+    /// Total number of bits this enumset uses. Note that the actual amount of space used is
+    /// rounded up to the next highest integer type (`u8`, `u16`, `u32`, `u64`, or `u128`).
+    pub fn bit_width() -> u8 {
+        T::VARIANT_COUNT as u8
+    }
+
+    /// Returns the raw bits of this set
+    pub fn to_bits(&self) -> u128 {
+        T::repr_to_u128(self.__enumset_underlying)
+    }
+    /// Constructs a bitset from raw bits
+    pub fn from_bits(bits: u128) -> Self {
+        EnumSet { __enumset_underlying: T::repr_from_u128(bits) }
+    }
+
     /// Returns the number of values in this set.
     pub fn len(&self) -> usize {
         T::count_ones(self.__enumset_underlying)
@@ -279,7 +304,6 @@ impl <T : EnumSetType> Iterator for EnumSetIter<T> {
 
 #[macro_export]
 #[doc(hidden)]
-#[cfg(feature = "nightly")]
 macro_rules! enum_set_type_internal_count_variants {
     ($next:ident ($($args:tt)*)
         $_00:ident $_01:ident $_02:ident $_03:ident $_04:ident $_05:ident $_06:ident $_07:ident
@@ -300,7 +324,7 @@ macro_rules! enum_set_type_internal_count_variants {
         $_f0:ident $_f1:ident $_f2:ident $_f3:ident $_f4:ident $_f5:ident $_f6:ident $_f7:ident
         $($rest:ident)+
     ) => {
-        ENUM_SET___TOO_MANY_ENUM_VARIANTS___MAX_IS_128
+        compile_error!("enum_set_type! can only accept up to 128 variants.")
     };
     ($next:ident ($($args:tt)*)
         $_00:ident $_01:ident $_02:ident $_03:ident $_04:ident $_05:ident $_06:ident $_07:ident
@@ -342,77 +366,6 @@ macro_rules! enum_set_type_internal_count_variants {
     };
 }
 
-#[macro_export]
-#[doc(hidden)]
-#[cfg(not(feature = "nightly"))]
-macro_rules! enum_set_type_internal_count_variants {
-    ($next:ident ($($args:tt)*)
-        $_00:ident $_01:ident $_02:ident $_03:ident $_04:ident $_05:ident $_06:ident $_07:ident
-        $_10:ident $_11:ident $_12:ident $_13:ident $_14:ident $_15:ident $_16:ident $_17:ident
-        $_20:ident $_21:ident $_22:ident $_23:ident $_24:ident $_25:ident $_26:ident $_27:ident
-        $_30:ident $_31:ident $_32:ident $_33:ident $_34:ident $_35:ident $_36:ident $_37:ident
-        $_40:ident $_41:ident $_42:ident $_43:ident $_44:ident $_45:ident $_46:ident $_47:ident
-        $_50:ident $_51:ident $_52:ident $_53:ident $_54:ident $_55:ident $_56:ident $_57:ident
-        $_60:ident $_61:ident $_62:ident $_63:ident $_64:ident $_65:ident $_66:ident $_67:ident
-        $_70:ident $_71:ident $_72:ident $_73:ident $_74:ident $_75:ident $_76:ident $_77:ident
-        $($rest:ident)+
-    ) => {
-        ENUM_SET___TOO_MANY_ENUM_VARIANTS___MAX_IS_64
-    };
-    ($next:ident ($($args:tt)*)
-        $_00:ident $_01:ident $_02:ident $_03:ident $_04:ident $_05:ident $_06:ident $_07:ident
-        $_10:ident $_11:ident $_12:ident $_13:ident $_14:ident $_15:ident $_16:ident $_17:ident
-        $_20:ident $_21:ident $_22:ident $_23:ident $_24:ident $_25:ident $_26:ident $_27:ident
-        $_30:ident $_31:ident $_32:ident $_33:ident $_34:ident $_35:ident $_36:ident $_37:ident
-        $($rest:ident)+
-    ) => {
-        enum_set_type_internal! { @$next u64 $($args)* }
-    };
-    ($next:ident ($($args:tt)*)
-        $_00:ident $_01:ident $_02:ident $_03:ident $_04:ident $_05:ident $_06:ident $_07:ident
-        $_10:ident $_11:ident $_12:ident $_13:ident $_14:ident $_15:ident $_16:ident $_17:ident
-        $($rest:ident)+
-    ) => {
-        enum_set_type_internal! { @$next u32 $($args)* }
-    };
-    ($next:ident ($($args:tt)*)
-        $_00:ident $_01:ident $_02:ident $_03:ident $_04:ident $_05:ident $_06:ident $_07:ident
-        $($rest:ident)+
-    ) => {
-        enum_set_type_internal! { @$next u16 $($args)* }
-    };
-    ($next:ident ($($args:tt)*) $($rest:ident)*) => {
-        enum_set_type_internal! { @$next u8 $($args)* }
-    };
-}
-
-#[macro_export]
-#[doc(hidden)]
-#[cfg(not(feature = "nightly"))]
-macro_rules! enum_set_type_nightly_impl {
-    ($enum_name:ident $repr:ident) => { }
-}
-
-#[macro_export]
-#[doc(hidden)]
-#[cfg(feature = "nightly")]
-#[allow_internal_unstable]
-macro_rules! enum_set_type_nightly_impl {
-    ($enum_name:ident $repr:ident) => {
-        impl $enum_name {
-            #[doc(hidden)]
-            pub const fn __enumset_construct(self, data: $repr) -> $crate::EnumSet<Self> {
-                $crate::EnumSet { __enumset_underlying: data }
-            }
-
-            #[doc(hidden)]
-            pub const fn __enumset_type_check(self, other: Self) -> Self {
-                other
-            }
-        }
-    }
-}
-
 #[macro_export]
 #[doc(hidden)]
 macro_rules! enum_set_type_internal {
@@ -446,8 +399,14 @@ macro_rules! enum_set_type_internal {
             fn from_u8(val: u8) -> Self {
                 unsafe { ::std::mem::transmute(val) }
             }
+
+            fn repr_to_u128(bits: Self::Repr) -> u128 {
+                bits as u128
+            }
+            fn repr_from_u128(bits: u128) -> Self::Repr {
+                bits as $repr
+            }
         }
-        enum_set_type_nightly_impl!($enum_name $repr);
         impl ::std::ops::BitOr<$enum_name> for $enum_name {
             type Output = $crate::EnumSet<$enum_name>;
             fn bitor(self, other: $enum_name) -> Self::Output {
@@ -540,42 +499,12 @@ macro_rules! enum_set_type {
     () => { };
 }
 
-/// Creates a EnumSet literal, which can be used in const contexts.
-///
-/// The format used is `enum_set!(Type, Type::A | Type::B | Type::C)`, where `Type` is the type of
-/// the variants that are expected to follow. When the `nightly` feature is enabled, that parameter
-/// can be omitted.
-///
-/// # Examples
-///
-/// ```rust
-/// # #[macro_use] extern crate enumset;
-/// # use enumset::*;
-/// # enum_set_type! {
-/// #     enum Enum { A, B, C }
-/// # }
-/// # fn main() {
-/// const CONST_SET: EnumSet<Enum> = enum_set!(Enum, Enum::A | Enum::B);
-/// assert_eq!(CONST_SET, Enum::A | Enum::B);
-/// # }
-/// ```
-#[macro_export]
-#[cfg(not(feature = "nightly"))]
-macro_rules! enum_set {
-    () => { EnumSet { __enumset_underlying: 0 } };
-    ($enum_name:ty, $($value:path)|* $(|)*) => {
-        $crate::EnumSet::<$enum_name> {
-            __enumset_underlying: 0 $(| (1 << ($value as $enum_name as u8)))*
-        }
-    }
-}
-
 /// Creates a EnumSet literal, which can be used in const contexts. The format used is
-/// `enum_set!(Type::A | Type::B | Type::C)`
+/// `enum_set!(Type::A | Type::B | Type::C)` Each variant must be of the same type, or
+/// a error will occur at compile-time.
 ///
 /// You may also explicitly state the type of the variants that follow, as in
-/// `enum_set!(Type, Type::A | Type::B | Type::C)`. This is the only form of this macro that
-/// is supported when the `nightly` feature is disabled.
+/// `enum_set!(Type, Type::A | Type::B | Type::C)`.
 ///
 /// # Examples
 ///
@@ -593,18 +522,30 @@ macro_rules! enum_set {
 /// assert_eq!(EXPLICIT_CONST_SET, Enum::A | Enum::B);
 /// # }
 /// ```
+///
+/// This macro is strongly typed. For example, the following will not compile:
+///
+/// ```compile_fail
+/// # #[macro_use] extern crate enumset;
+/// # use enumset::*;
+/// # enum_set_type! {
+/// #     enum Enum { A, B, C }
+/// #     enum Enum2 { A, B, C }
+/// # }
+/// # fn main() {
+/// let type_error = enum_set!(Enum::A | Enum2::B);
+/// # }
+/// ```
 #[macro_export]
-#[cfg(feature = "nightly")]
 macro_rules! enum_set {
     () => { EnumSet::new() };
-    ($value:path $(|)*) => {
-        $value.__enumset_construct(1 << ($value as u8))
-    };
-    ($first_value:path | $($value:path)|* $(|)*) => {
-        $first_value.__enumset_construct(
-            (1 << ($first_value as u8))
-            $(| (1 << ($first_value.__enumset_type_check($value) as u8)))*
-        )
+    ($($value:path)|* $(|)*) => {
+        $crate::EnumSetSameTypeHack {
+            unified: &[$($value,)*],
+            enum_set: $crate::EnumSet {
+                __enumset_underlying: 0 $(| (1 << ($value as u8)))*
+            },
+        }.enum_set
     };
     ($enum_name:ty, $($value:path)|* $(|)*) => {
         $crate::EnumSet::<$enum_name> {
@@ -627,7 +568,6 @@ mod test {
         }
     }
 
-    #[cfg(feature = "nightly")]
     enum_set_type! {
         enum LargeEnum {
             _00,  _01,  _02,  _03,  _04,  _05,  _06,  _07,
@@ -658,7 +598,6 @@ mod test {
         A, B, C, D, E, F, G, H, I, J, K, L, M, N, O, P, Q, R, S, T, U, V, W, X, Y, Z,
     }
 
-    #[cfg(feature = "nightly")]
     test_variants! { LargeEnum large_enum_variant_range_test
         _00,  _01,  _02,  _03,  _04,  _05,  _06,  _07,
         _10,  _11,  _12,  _13,  _14,  _15,  _16,  _17,
@@ -753,6 +692,5 @@ mod test {
     }
 
     test_enum!(Enum, small_enum);
-    #[cfg(feature = "nightly")]
     test_enum!(LargeEnum, large_enum);
 }
